<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davis@davisonpro.dev | https://davisonpro.dev>
 * @copyright  2020 Proxim
 * @version    1.0
 * @since      File available since Release 1.0
*/

namespace Proxim\Presenter\Order;

use Proxim\Presenter\AbstractLazyArray;
use Doctrine\Common\Annotations\AnnotationException;
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Order\Order;
use Proxim\Preference\Discipline;
use Proxim\Preference\PaperFormat;
use Proxim\Preference\PaperType;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;

class CalculationsOrderLazyArray extends AbstractLazyArray {
    /** @var Application */
    public $application;

    /** @var Order */
    public $order;

    /** @var Currency */
    public $currency;

    /** @var array $orderFiles */
    public $orderFiles = array();

    /** @var array $customerFiles */
    public $customerFiles = array();

    /** @var array $writerFiles */
    public $writerFiles = array();

    /** @var array $pdSchedule */
    public $pdSchedule = array();

    /** @var array $customerFilesCount */
    public $customerFilesCount = 0;

    /** @var array $writerFilesCount */
    public $writerFilesCount = 0;

    /** @var array $orderFilesCount */
    public $orderFilesCount = 0;

    /** @var array $unseenFilesCount */
    public $unseenFilesCount = 0;

    /** @var array $allMessagesCount */
    public $allMessagesCount = 0;

    /** @var array $countUnreadMessages */
    public $countUnreadMessages = 0;

    /** @var bool $latestFileIsDownloaded */
    public $latestFileIsDownloaded = false;

    /**
     * OrderArray constructor.
     *
     * @throws AnnotationException
     * @throws ReflectionException
     */
    public function __construct(Order $order)
    {
        $this->application = Application::getInstance();
        $this->order = $order;

        $currency = new Currency( (int) $order->currency_id );

        if(!Validate::isLoadedObject($currency)) {
            $default_currency_id = (int) Configuration::get('CURRENCY_DEFAULT');
            $currency = new Currency( $default_currency_id );
        }

        $this->currency = $currency;

        /** Order Messages */
        $orderMessages = $order->getMessages();

        $countUnreadMessages = 0;
        foreach( $orderMessages as $message ) {
            if ( 
                $message->sender_id != $order->customer_id && 
                !$message->is_viewed
            ) {
                $countUnreadMessages++;
            }
        }

        $this->countUnreadMessages = $countUnreadMessages;
        $this->allMessagesCount = count($orderMessages);


        $orderFiles = $order->getFiles();
        $writerFiles = $customerFiles = array();

        $customerFilesCount = $writerFilesCount = $unseenFilesCount = 0;

        foreach( $orderFiles as $file ) {
            // customer files
            if ($file->uploader_id == $order->customer_id) {
                $customerFiles[] = $file;
                $customerFilesCount++;

            // writer files
            } else {
                if( $this->order->status_id == Order::DELIVERED ||
                    $this->order->status_id == Order::FINISHED 
                ) {
                    // unseen files
                    if ( !$file->downloaded ) {
                        $unseenFilesCount++;
                    }

                    $writerFiles[] = $file;
                    $writerFilesCount++;
                }
                
            }
        }

        if ( $writerFilesCount > 0 ) {
            $lastFile = ArrayUtils::shift($writerFiles);
            if( Validate::isLoadedObject($lastFile) ) {
                $this->latestFileIsDownloaded = (bool) $lastFile->downloaded;
            }
        }

        $this->orderFilesCount = $customerFilesCount+$writerFilesCount;
        $this->customerFilesCount = $customerFilesCount;
        $this->writerFilesCount = $writerFilesCount;
        $this->unseenFilesCount = $unseenFilesCount;

        $this->customerFiles = $customerFiles;
        $this->writerFiles = $writerFiles;

        $this->orderFiles = $orderFiles;

        parent::__construct();
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getId()
    {
        return (int) $this->order->id;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getParentId()
    {
        return (int) $this->order->parent_id;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getCustomerId()
    {
        return (int) $this->order->customer_id;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getCurrencyId()
    {
        return (int) $this->currency->id;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getCurrencyRate()
    {
        return (float) $this->currency->conversion_rate;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getRequestedWriterId()
    {
        return (int) $this->order->requested_writer_id;
    }
    
    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getServiceTypeId()
    {
        return (int) $this->order->service_type;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getServiceTypeTitle()
    {
        global $globals;
        $service_type = ArrayUtils::get($globals['serviceTypes'], (int) $this->order->service_type);
        return (string) ArrayUtils::get($service_type, 'title');
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getStatusId()
    {
        return (int) $this->order->status_id;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getStatusTitle()
    {
        global $globals;
        $status = ArrayUtils::get($globals['orderStatuses'], (int) $this->order->status_id);
        return (string) ArrayUtils::get($status, 'title');
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getStatusDescription()
    {
        global $globals;
        $status = ArrayUtils::get($globals['orderStatuses'], (int) $this->order->status_id);
        return (string) ArrayUtils::get($status, 'description');
    }
  
    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getDisciplineId() {
        return (int) $this->order->topic_category_id;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getDisciplineTitle() {
        if($this->order->topic_category_option) {
            return $this->order->topic_category_option;
        }

        $discipline = new Discipline( (int) $this->order->topic_category_id);
        return (string) $discipline->title;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getTitle()
    {
        return (string) $this->getDisciplineTitle();
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getInstructions() {
        return (string) $this->order->paper_details;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getComments() {
        return (string) $this->order->comments;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getSoftware() {
        return (string) $this->order->software;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getDeadlineId() {
        return (int) $this->order->tariff_id;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getTariffHrs() {
        return (int) $this->order->hrs_customer;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getSizeId() {
        return (int) $this->order->task_size_id;
    }
    
    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getSizeTitle() {
        global $globals;

        $taskSize = ArrayUtils::get($globals['taskSizes'], $this->order->task_size_id, []);
        return (string) ArrayUtils::get($taskSize, 'title');
    }

    /**
     * @arrayAccess
     *
     * @return float
     */
    public function getPrice() {
        return (float) $this->order->total;
    }

    /**
     * @arrayAccess
     *
     * @return float
     */
    public function getTax() {
        return (float) $this->order->tax;
    }

    /**
     * @arrayAccess
     *
     * @return float
     */
    public function getNoDiscountPrice() {
        return (float) $this->order->raw_cost;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getCouponType() {
        return (int) $this->order->discount_type;
    }

    /**
     * @arrayAccess
     *
     * @return float
     */
    public function getCouponValue() {
        return (float) $this->order->discount;
    }

    /**
     * @arrayAccess
     *
     * @return array
     */
    public function getAdditionalPayments() {
        return $this->order->getAdditionalPayments();
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getAllFilesCount() {
        return (int) $this->orderFilesCount;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getNewFilesCount() {
        return (int) $this->unseenFilesCount;
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getWithAdditionalMaterials() {
        return count($this->customerFiles) > 0 ? true : false;
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getLatestFileIsDownloaded() {
        return (bool) $this->latestFileIsDownloaded;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getAllMessagesCount() {
        return (int) $this->allMessagesCount;
    }

    /**
     * @arrayAccess
     *
     * @return int
     */
    public function getNewMessagesCount() {
        return (int) $this->countUnreadMessages;
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getHasReassignments() {
        return false;
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getIsWriterAvailableForRevision() {
        return false;
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getFullRefundAllowed() {
        return (bool) Configuration::get('REQUEST_REFUND_ALLOWED');
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getIsAvailableTrustpilot() {
        return true;
        return (bool) Configuration::get('TRUST_PILOT_ENABLED');
    }

    /**
     * @arrayAccess
     *
     * @return bool
     */
    public function getIsLeaveTipButtonVisible() {
        return (bool) $this->order->leaveTipButtonVisible();
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getCancelledAt() {
        if( $this->order->status_id == Order::CANCELLED ) {
            return DateUtils::convertToISOFormat($this->order->cancelled_at);
        }

        return;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getApprovedAt() {
        if( $this->order->status_id == Order::FINISHED ) {
            return DateUtils::convertToISOFormat($this->order->approved_at);
        }

        return;
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getRealDeadline() {
        return DateUtils::convertToISOFormat($this->order->customer_deadline);
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getWhenIt() {
        return DateUtils::convertToISOFormat($this->order->customer_deadline);
    }

    /**
     * @arrayAccess
     *
     * @return string
     */
    public function getCreatedAt() {
        return DateUtils::convertToISOFormat($this->order->date_add);
    }
}