<?php

use Proxim\Application;
use Proxim\Configuration;
use Proxim\Coupon;
use Proxim\Module\Module;
use Proxim\Order\Order;
use Proxim\Tools;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;


define('CURRENT_AFF_MODULE_DIR', realpath(dirname(__FILE__)));

require_once(CURRENT_AFF_MODULE_DIR . '/classes/Affiliate.php');

class Affiliates extends Module
{
    const AFFILIATE_LANDING_URL = 'AFFILIATE_LANDING_URL';
    const AFFILIATE_DASHBOARD_TITLE = 'AFFILIATE_DASHBOARD_TITLE';
    const AFFILIATE_LIFETIME_COMMISSION = 'AFFILIATE_LIFETIME_COMMISSION';
    const AFFILIATE_COMMISSION_TYPE = 'AFFILIATE_COMMISSION_TYPE';
    const AFFILIATE_COMMISSION_PERCENTAGE = 'AFFILIATE_COMMISSION_PERCENTAGE';
    const AFFILIATE_COMMISSION_AMOUNT = 'AFFILIATE_COMMISSION_AMOUNT';
    const AFFILIATE_MINUMUM_WITHRAWAL = 'AFFILIATE_MINUMUM_WITHRAWAL';
    const AFFILIATE_DISCOUNT_TYPE = 'AFFILIATE_DISCOUNT_TYPE';
    const AFFILIATE_DISCOUNT_PERCENTAGE = 'AFFILIATE_DISCOUNT_PERCENTAGE';
    const AFFILIATE_DISCOUNT_AMOUNT = 'AFFILIATE_DISCOUNT_AMOUNT';

    public function __construct()
    {
        $this->name = 'affiliates';
        $this->icon = 'fas fa-bullhorn';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'Affiliates';
        $this->description = 'Affiliate dashboard for tracking affiliate programs & networks';
    }

    public function getDomain($url) {
        return parse_url($url, PHP_URL_HOST) ?? null;
    }

    public function hookActionBeforeInitAffiliate() {
        $user = $this->application->container->user;

        if(!$user->isLogged() && Tools::getValue('aff') ) {
            $affiliate_id = (int) Tools::getValue('aff');

            if(!isset($_SESSION['affiliate_id'])) {
                $affiliate = new Affiliate( $affiliate_id );
                if (Validate::isLoadedObject($affiliate)) {

                    $referral_url = isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : "";

                    Db::getInstance()->insert('affiliate_click',[
                        'affiliate_id' => $affiliate->id,
                        'origin' => $referral_url,
                        'origin_base' => $this->getDomain($referral_url),
                        'date_add' => DateUtils::now()
                    ]);

                    $click_id = Db::getInstance()->Insert_ID();

                    $_SESSION['affiliate_id'] = (int) $affiliate->id;
                    $_SESSION['click_id'] = (int) $click_id;
                }
            } 
        }
    }

    public function hookActionCustomerAccountAdd($payload) {
        $newCustomer = ArrayUtils::get($payload, 'newCustomer');

        if(isset($_SESSION['click_id']) && isset($_SESSION['affiliate_id'])) {
            $click_id = (int) $_SESSION['click_id'];
            $affiliate_id = (int) $_SESSION['affiliate_id'];

            // add customer as a click
            Db::getInstance()->update('affiliate_click',[
                    'customer_id' => $newCustomer->id,
                    'date_upd' => DateUtils::now()
                ],
                'click_id = ' . (int) $click_id . ' AND affiliate_id = ' . $affiliate_id
            );

            /** create a coupon for the customer */
            $configCouponType = Configuration::get(self::AFFILIATE_DISCOUNT_TYPE, PROX_ADMIN_SITE_ID, 'amount');
            $configCouponPercentage = (float) Configuration::get(self::AFFILIATE_DISCOUNT_PERCENTAGE, PROX_ADMIN_SITE_ID, 10);
            $configCouponAmount = (float) Configuration::get(self::AFFILIATE_DISCOUNT_AMOUNT, PROX_ADMIN_SITE_ID, 10);

            $couponType = $configCouponType == 'amount' ? Coupon::AMOUNT: Coupon::PERCENT;
            $couponValue = $configCouponType == 'amount' ? $configCouponAmount: $configCouponPercentage;

            Db::getInstance()->insert('coupon', [
                'user_id' => $newCustomer->id,
                'site_id' => $newCustomer->site_id,
                'coupon_code' => Tools::randomGen(6),
                'coupon_type' => $couponType,
                'coupon_value' => $couponValue,
                'expiry' => DateUtils::date(strtotime('+30 days')),
                'active' => 1,
                'date_add' => DateUtils::now()
            ]);

            unset($_SESSION['affiliate_id']);
            unset($_SESSION['click_id']);
        }
    }

    public function hookActionAfterOrderPayment( $payload ) {
        $order = ArrayUtils::get($payload, 'order');

        if(Validate::isLoadedObject($order)) {
            $referral = Db::getInstance()->getRow('SELECT * FROM ' . Db::prefix('affiliate_click') . ' AS ac WHERE ac.customer_id = ' . (int) $order->customer_id);

            if($referral) {
                // check if affiliate should receive the commission. 
                $lifetimeCommission = (bool) Configuration::get(self::AFFILIATE_LIFETIME_COMMISSION, PROX_ADMIN_SITE_ID, false);
                $exisitingReferals = Db::getInstance()->executeS('SELECT * FROM ' . Db::prefix('affiliate_referral') . ' AS ar WHERE ar.customer_id = ' . (int) $order->customer_id);

                if(!count($exisitingReferals) || $lifetimeCommission) {

                    // check if order exists
                    $exisitingOrder = Db::getInstance()->getRow('SELECT * FROM ' . Db::prefix('affiliate_referral') . ' AS ar WHERE ar.order_id = ' . (int) $order->id);
                    if(!$exisitingOrder) {
                        // calculate commission
                        $commissionType = Configuration::get(self::AFFILIATE_COMMISSION_TYPE, PROX_ADMIN_SITE_ID, 'amount');
                        $commissionPercentage = Configuration::get(self::AFFILIATE_COMMISSION_PERCENTAGE, PROX_ADMIN_SITE_ID, 10);
                        $commissionAmount = Configuration::get(self::AFFILIATE_COMMISSION_AMOUNT, PROX_ADMIN_SITE_ID, 10);

                        $referralAmount = 0;

                        if ($commissionType == "amount") {
                            $referralAmount = (float) $commissionAmount;
                        } else { 
                            $referralAmount = self::normalizePrice(
                                ($order->total_paid * $commissionPercentage) / 100
                            );
                        } 

                        // insert referal
                        Db::getInstance()->insert('affiliate_referral',[
                            'affiliate_id' => (int) $referral['affiliate_id'],
                            'customer_id' => (int) $referral['customer_id'],
                            'order_id' => (int) $order->id,
                            'amount' => (float) $referralAmount,
                            'date_add' => DateUtils::now()
                        ]);

                        // update to referral is converted
                        Db::getInstance()->update('affiliate_click',[
                                'is_converted' => true,
                                'date_upd' => DateUtils::now()
                            ],
                            'customer_id = ' . (int) $referral['customer_id'] . ' AND affiliate_id = ' . $referral['affiliate_id']
                        );
                    }

                    // delete the generated coupon
                    Db::getInstance()->delete('coupon', 'coupon_id = ' . (int) $order->coupon_id . ' AND user_id = ' . (int) $order->customer_id );
                }
            }
        }
    }

    /**
     * Approximates number to hundredth
     * @param {number} amount
     * @returns {number}
     */
    public static function normalizePrice( $amount ) {
        return round($amount * 100) / 100;
    }  
}