<?php

use Proxim\Application;
use Proxim\Configuration;
use Proxim\Cookie;
use Proxim\Crypto\Hashing;
use Proxim\Database\DbQuery;
use Proxim\Module\Module;
use Proxim\Pager;
use Proxim\ObjectModel;
use Proxim\Presenter\Order\OrderPresenter;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\User\Customer;
use Proxim\User\Employee;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;
use React\EventLoop\Factory;
use React\Http\Browser;

define('CURRENT_DATALAYER_MODULE_DIR', realpath(dirname(__FILE__)));

require_once CURRENT_DATALAYER_MODULE_DIR . '/vendor/autoload.php';

class DataLayer extends Module
{
    const DATALAYER_ENABLED = 'DATALAYER_ENABLED';
    const DATALAYER_HOST = 'DATALAYER_HOST';

    public function __construct()
    {
        $this->name = 'datalayer';
        $this->icon = 'fas fa-user-clock';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'Data Layer';
        $this->description = 'Manage all orders from one dashboard';
    }

    public function getHost($url) {
        $parsed_url = parse_url($url);

        if (!$parsed_url || !isset($parsed_url['host'])) {
            return false;
        }

        $host = $parsed_url['host'];

        return $host;
    }

    public function makeRequest($action, $data)
    {
        $client_host = $this->getHost(Configuration::get('SITE_DOMAIN'));
        if(!$client_host) {
            return;
        }

        $admin_host = $this->getHost(Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID));
        if(!$admin_host) {
            return;
        }

        $datalayer_domain = Configuration::get(self::DATALAYER_HOST, PROX_ADMIN_SITE_ID);
        $url = "{$datalayer_domain}/api/datalayer/sync";

        $loop = React\EventLoop\Loop::get();
        $browser = new Browser($loop);

        $payload = array(
            'action' => $action,
            'object_id' => (int) $data['id'],
            'object' => $data,
            'domain' => $client_host,
            'admin_domain' => $admin_host,
        );

        $browser->post($url, [
            'Content-Type' => 'application/x-www-form-urlencoded'
        ], http_build_query($payload))->then(
            function (\Psr\Http\Message\ResponseInterface $response) {
                // echo "Response from server: " . $response->getBody();
            },
            function (Exception $exception) {
                // echo "Request failed: " . $exception->getMessage();
            }
        );

        // Start the event loop
        $loop->run();
    }

    public function presentOrder($order)
    {
        $objectPresenter = new OrderPresenter();
        $orderPresented = $objectPresenter->present($order);

        $data = array(
            'id' => (int) $order->id,
            'currency_id' => (int) $order->currency_id,
            'currency_rate' => (float) $order->currency_to_usd_rate,
            'status_id' => (int) $order->status_id,
            'service_type_id' => (int) $order->service_type,
            'task_size_id' => (int) $order->task_size_id,
            'academic_level' => $orderPresented['academicLevelTitle'],
            'paper_type' => $orderPresented['paperTypeTitle'],
            'topic_category' => $orderPresented['topicCategory'],
            'paper_format' => $orderPresented['paperFormatTitle'],
            'title' => $orderPresented['title'],
            'spacing' => $order->spacing,
            'pages' => (int) $order->pages,
            'words' => (int) $order->words,
            'charts' => (int) $order->charts,
            'slides' => (int) $order->slides,
            'excel_sheets' => (int) $order->charts,
            'customer_deadline' => $order->customer_deadline,
            'writer_deadline' => $order->writer_deadline,
            'total' => (float) $order->total,
            'discount' => (float) $order->discount,
            'date_add' => $order->date_add,
        );

        return $data;
    }

    public function presentMessage($message)
    {
        $data = array(
            'id' => $message->id,
            'order_id' => $message->order_id,
            'from_department' => $message->from_department,
            'to_department' => $message->to_department,
            'subject' => $message->subject,
            'body' => $message->body,
            'is_viewed' => (bool) $message->is_viewed,
            'date_add' => $message->date_add,
        );

        return $data;
    }

    public function hookActionObjectProximOrderOrderAddAfter($params)
    {
        $order = ArrayUtils::get($params, 'object');

        if(!Configuration::get(self::DATALAYER_ENABLED, PROX_ADMIN_SITE_ID)) {
            return;
        }

        if (!Validate::isLoadedObject($order)) {
            return;
        }

        $data = $this->presentOrder($order);

        $this->makeRequest('OrderAdd', $data);
    }

    public function hookActionObjectProximOrderOrderUpdateAfter($params)
    {
        $order = ArrayUtils::get($params, 'object');

        if(!Configuration::get(self::DATALAYER_ENABLED, PROX_ADMIN_SITE_ID)) {
            return;
        }

        if (!Validate::isLoadedObject($order)) {
            return;
        }

        $data = $this->presentOrder($order);

        $this->makeRequest('OrderUpdate', $data);
    }

    public function hookActionObjectProximOrderOrderMessageAddAfter($params)
    {
        $message = ArrayUtils::get($params, 'object');

        if(!Configuration::get(self::DATALAYER_ENABLED, PROX_ADMIN_SITE_ID)) {
            return;
        }

        if (!Validate::isLoadedObject($message)) {
            return;
        }

        $data = $this->presentMessage($message);

        $this->makeRequest('MessageAdd', $data);
    }

    public function hookActionObjectProximOrderOrderMessageUpdateAfter($params)
    {
        $message = ArrayUtils::get($params, 'object');

        if(!Configuration::get(self::DATALAYER_ENABLED, PROX_ADMIN_SITE_ID)) {
            return;
        }

        if (!Validate::isLoadedObject($message)) {
            return;
        }

        $data = $this->presentMessage($message);

        $this->makeRequest('MessageUpdate', $data);
    }

    public function hookActionObjectProximOrderOrderMessageDeleteAfter($params)
    {
        $message = ArrayUtils::get($params, 'object');

        if(!Configuration::get(self::DATALAYER_ENABLED, PROX_ADMIN_SITE_ID)) {
            return;
        }
    }
}
