<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;
use SendGrid\Mail\To;

class EazzyPay extends PaymentModule
{

    /** @var string $configured_token_environment */
    public $configured_token_environment;

    /** @var string $configured_launch_environment */
    public $configured_launch_environment;

    /**
    * EazzyPay configuration
    */
    const EAZZY_CHECKOUT_ENABLED = 'EAZZY_CHECKOUT_ENABLED';
    const EAZZY_CHECKOUT_MODE = 'EAZZY_CHECKOUT_MODE';
    const EAZZY_CHECKOUT_MERCHANT_CODE = 'EAZZY_CHECKOUT_MERCHANT_CODE';
    const EAZZY_CHECKOUT_MERCHANT_NAME = 'EAZZY_CHECKOUT_MERCHANT_NAME';
    const EAZZY_CHECKOUT_OUTLET_CODE = 'EAZZY_CHECKOUT_OUTLET_CODE';
    const EAZZY_CHECKOUT_API_KEY = 'EAZZY_CHECKOUT_API_KEY';
    const EAZZY_CHECKOUT_API_PASSWORD = 'EAZZY_CHECKOUT_API_PASSWORD';

    public function __construct()
    {
        $this->name = 'eazzypay';
        $this->icon = 'fa fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'EazzyPay';
        $this->description = 'Set up EazzyPay as a payment method on your website';

        $this->bootstrap = true;

        if (Configuration::get('EAZZY_CHECKOUT_MODE') == 'sandbox'){
            $this->configured_token_environment = 'https://api-test.equitybankgroup.com/v1/token';
            $this->configured_launch_environment = 'https://api-test.equitybankgroup.com/v2/checkout/launch';
        } else {
            $this->configured_token_environment = 'https://api.equitybankgroup.com/v1/token';
            $this->configured_launch_environment = 'https://api.equitybankgroup.com/v2/checkout/launch';
        }

        parent::__construct();
    }

    public function checkAccess() {
        $user = $this->application->container->user;
        return $user->is_admin === true;
    }

    public function isActive() {
        return Configuration::get(self::EAZZY_CHECKOUT_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function getToken() {
        // Payload Details
        $payload = array(
            'merchantCode' 	=> Configuration::get(self::EAZZY_CHECKOUT_MERCHANT_CODE),
            'password' 	=> Configuration::get(self::EAZZY_CHECKOUT_API_PASSWORD),
            'grant_type' => 'password',
        );

        $ch = curl_init($this->configured_token_environment );
        curl_setopt($ch, CURLOPT_URL, $this->configured_token_environment );
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: " . Configuration::get(self::EAZZY_CHECKOUT_API_KEY),
            "Content-Type: application/x-www-form-urlencoded",
        ));
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($payload));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $response = Tools::jsonDecode($response, true);

        //  return $response;
        if ( !ArrayUtils::get($response, 'payment-token') ) {
            throw new Exception('Jenga PGW Response did not get any data.', 'jpgw');
        }

        return ArrayUtils::get($response, 'payment-token');
    }

    public function setVars() {
        $app = $this->application;
        $user = $app->container->user;
        $smarty = $this->smarty;
        
        $eazzypay_logo = Configuration::get('UPLOADS_PATH') . '/' . Configuration::get('SITE_EMAIL_LOGO');
        $eazzypay_token = $this->getToken();

        $smarty->assign([
            'eazzypay_logo' => $eazzypay_logo,
            'eazzypay_launch_environment' => $this->configured_launch_environment,
            'eazzypay_token' => $eazzypay_token,
            'eazzypay_merchant_code' => Configuration::get( self::EAZZY_CHECKOUT_MERCHANT_CODE ),
            'eazzypay_merchant_name' => Configuration::get( self::EAZZY_CHECKOUT_MERCHANT_NAME ),
            'eazzypay_outlet_code' => Configuration::get( self::EAZZY_CHECKOUT_OUTLET_CODE ),
            'payee_email' => $user->email
        ]);
    }

    public function processOrderCheckout($order, $currency) {
        $this->setVars();
        return $this->getTemplateContent('eazzypay.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $this->setVars();
        return $this->getTemplateContent('eazzypay.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $this->setVars();
        return $this->getTemplateContent('eazzypay.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $this->setVars();
        return $this->getTemplateContent('eazzypay.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $this->setVars();
        return $this->getTemplateContent('eazzypay.tip');
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    {
        $app = $this->application;
        $user = $app->container->user;

        $handle = ArrayUtils::get($params, 'handle');
        $status = ArrayUtils::get($params, 'status');
        $transactionId = ArrayUtils::get($params, 'transactionId');
        $amount = ArrayUtils::get($params, 'amount');
        $merchantOrderRef = ArrayUtils::get($params, 'merchantOrderRef');

        if( $status == 'paid' ) {
            $params['nodeId'] = $merchantOrderRef;
            $params['payment_method'] = $this->name;
            $params['paymentId'] = $transactionId;
            $params['amount'] = $amount;
            $params['customer_id'] = $user->id;

            PaymentModule::processPayment( $merchantOrderRef, $handle, $params );
        }

    }
}