<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

require_once dirname(__FILE__) . '/vendor/autoload.php';

class FasterPay extends PaymentModule
{
    /**
    * FasterPay configuration
    */
    const FASTERPAY_ENABLED = 'FASTERPAY_ENABLED';
    const FASTERPAY_MODE = 'FASTERPAY_MODE';
    const FASTERPAY_API_PUBLIC_KEY = 'FASTERPAY_API_PUBLIC_KEY';
    const FASTERPAY_API_PRIVATE_KEY = 'FASTERPAY_API_PRIVATE_KEY';

    public function __construct()
    {
        $this->name = 'fasterpay';
        $this->icon = 'fab fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'FasterPay';
        $this->description = 'FasterPay allows you to accept payment on your site via credit card';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::FASTERPAY_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest,
        ]);

        $options['currency'] = array(
            'symbol' => $currency->symbol,
            'rate' => $currency->conversion_rate
        );

        $redirectUri = $app->base_uri . '/dashboard/order/' . $order->id;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?redirectUri=' . $redirectUri;
        $pingback_url = $app->base_uri . '/api/payment/'.$this->name.'/callback?handle=order&nodeId='.$order->id;

        $form = $gateway->paymentForm()->buildForm(
            [
                'description' => "Payment of " . formatPrice($order->total, $options),
                'amount' => $order->total,
                'currency' => $currency->iso_code,
                'merchant_order_id' => time(),
                'success_url' => $success_url,
                'pingback_url' => $pingback_url,
                'sign_version' => 'v2' // to use version 1 please skip this param or set it 'v1'
            ],
            [
                'autoSubmit' => false,
                'hidePayButton' => true
            ]
        );

        $smarty->assign([
            'base_uri' => $app->base_uri,
            '_form' => $form,
            'redirectUri' => $redirectUri
        ]);

        return $this->getTemplateContent('fasterpay.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest,
        ]);

        $options['currency'] = array(
            'symbol' => $currency->symbol,
            'rate' => $currency->conversion_rate
        );

        $redirectUri = $app->base_uri . '/dashboard/orders';
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?redirectUri=' . $redirectUri;
        $pingback_url = $app->base_uri . '/api/payment/'.$this->name.'/callback?handle=paper&nodeId='.$paper->id.'&customer_id='.$user->id;

        $form = $gateway->paymentForm()->buildForm(
            [
                'description' => "Payment of " . formatPrice($paper->price, $options),
                'amount' => $paper->price,
                'currency' => $currency->iso_code,
                'merchant_order_id' => time(),
                'success_url' => $success_url,
                'pingback_url' => $pingback_url,
                'sign_version' => 'v2' // to use version 1 please skip this param or set it 'v1'
            ],
            [
                'autoSubmit' => false,
                'hidePayButton' => true
            ]
        );

        $smarty->assign([
            'base_uri' => $app->base_uri,
            '_form' => $form,
            'redirectUri' => $redirectUri
        ]);

        return $this->getTemplateContent('fasterpay.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest,
        ]);

        $options['currency'] = array(
            'symbol' => $currency->symbol,
            'rate' => $currency->conversion_rate
        );

        $redirectUri = $app->base_uri . '/dashboard/order/' . $addPayment->order_id;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?redirectUri=' . $redirectUri;
        $pingback_url = $app->base_uri . '/api/payment/'.$this->name.'/callback?handle=add-payment&nodeId='.$addPayment->order_id;

        $form = $gateway->paymentForm()->buildForm(
            [
                'description' => "Payment of " . formatPrice($addPayment->total, $options),
                'amount' => $addPayment->total,
                'currency' => $currency->iso_code,
                'merchant_order_id' => time(),
                'success_url' => $success_url,
                'pingback_url' => $pingback_url,
                'sign_version' => 'v2' // to use version 1 please skip this param or set it 'v1'
            ],
            [
                'autoSubmit' => false,
                'hidePayButton' => true
            ]
        );

        $smarty->assign([
            'base_uri' => $app->base_uri,
            '_form' => $form,
            'redirectUri' => $redirectUri
        ]);

        return $this->getTemplateContent('fasterpay.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest,
        ]);

        $options['currency'] = array(
            'symbol' => $currency->symbol,
            'rate' => $currency->conversion_rate
        );

        $redirectUri = $app->base_uri . '/dashboard/wallet';
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?redirectUri=' . $redirectUri;
        $pingback_url = $app->base_uri . '/api/payment/'.$this->name.'/callback?handle=wallet&nodeId='.$user_id;

        $form = $gateway->paymentForm()->buildForm(
            [
                'description' => "Payment of " . formatPrice($amount, $options),
                'amount' => $amount,
                'currency' => $currency->iso_code,
                'merchant_order_id' => time(),
                'success_url' => $success_url,
                'pingback_url' => $pingback_url,
                'sign_version' => 'v2' // to use version 1 please skip this param or set it 'v1'
            ],
            [
                'autoSubmit' => false,
                'hidePayButton' => true
            ]
        );

        $smarty->assign([
            'base_uri' => $app->base_uri,
            '_form' => $form,
            'redirectUri' => $redirectUri
        ]);

        return $this->getTemplateContent('fasterpay.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest,
        ]);

        $options['currency'] = array(
            'symbol' => $currency->symbol,
            'rate' => $currency->conversion_rate
        );

        $redirectUri = $app->base_uri . '/dashboard/order/' . $order_id;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?redirectUri=' . $redirectUri;
        $pingback_url = $app->base_uri . '/api/payment/'.$this->name.'/callback?handle=tip&nodeId='.$order_id;

        $form = $gateway->paymentForm()->buildForm(
            [
                'description' => "Payment of " . formatPrice($amount, $options),
                'amount' => $amount,
                'currency' => $currency->iso_code,
                'merchant_order_id' => time(),
                'success_url' => $success_url,
                'pingback_url' => $pingback_url,
                'sign_version' => 'v2' // to use version 1 please skip this param or set it 'v1'
            ],
            [
                'autoSubmit' => false,
                'hidePayButton' => true
            ]
        );

        $smarty->assign([
            'base_uri' => $app->base_uri,
            '_form' => $form,
            'redirectUri' => $redirectUri
        ]);
        
        return $this->getTemplateContent('fasterpay.tip');
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPayment($node_id, $handle, $payload) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;

        $publicKey = Configuration::get(self::FASTERPAY_API_PUBLIC_KEY);
        $privateKey = Configuration::get(self::FASTERPAY_API_PRIVATE_KEY);
        $isTest = Configuration::get(self::FASTERPAY_MODE) == 'sandbox' ? true : false;

        $gateway = new FasterPay\Gateway([
            'publicKey' => $publicKey,
            'privateKey' => $privateKey,
            'isTest' => $isTest
        ]);

        $signVersion = FasterPay\Services\Signature::SIGN_VERSION_1;
        if (!empty($_SERVER['HTTP_X_FASTERPAY_SIGNATURE_VERSION'])) {
            $signVersion = $_SERVER['HTTP_X_FASTERPAY_SIGNATURE_VERSION'];
        }

        $pingbackData = $validationParams = [];
        switch ($signVersion) {
            case FasterPay\Services\Signature::SIGN_VERSION_1:
                $validationParams = ["apiKey" => $_SERVER["HTTP_X_APIKEY"]];
                $pingbackData = $_REQUEST;
                break;

            case FasterPay\Services\Signature::SIGN_VERSION_2:
                $validationParams = [
                    'pingbackData' => file_get_contents('php://input'),
                    'signVersion' => $signVersion,
                    'signature' => $_SERVER["HTTP_X_FASTERPAY_SIGNATURE"],
                ];
                $pingbackData = json_decode(file_get_contents('php://input'), 1);
                break;

            default:
                exit('NOK');
        }

        if (empty($pingbackData)) {
            exit('NOK');
        }

        if (!$gateway->pingback()->validate($validationParams)) {
            exit('NOK');
        }

        $paymentOrder = ArrayUtils::get($pingbackData, 'payment_order');
        $id = ArrayUtils::get($paymentOrder, 'id');
        $merchant_order_id = ArrayUtils::get($paymentOrder, 'merchant_order_id');
        $paid_currency = ArrayUtils::get($paymentOrder, 'paid_currency');
        $paid_amount = ArrayUtils::get($paymentOrder, 'paid_amount');
        $status = ArrayUtils::get($paymentOrder, 'status');
        $userDetails = ArrayUtils::get($paymentOrder, 'user');
        $userEmail = ArrayUtils::get($userDetails, 'email');

        if($status == 'successful') {
            $payload['paymentId'] = $id;
            $payload['currency'] = $paid_currency;
            $payload['amount'] = toFixed($paid_amount);

            PaymentModule::processPayment( $node_id, $handle, $payload );

            $paymentPayload = array(
                "payment_order_id" => $id,
                "merchant_reference_id" => $merchant_order_id,
                "sign_version" => "v2",
                "status" => "consumed",
                "refundable" => 0,
                "notify_user" => 0,
                "details" => "Payment of " . formatPrice($paid_amount),
                "shipping_address" => array(
                  "email" => $userEmail
                ),
                "type" => "digital",
                "public_key" => $gateway->getConfig()->getPublicKey(),
            );

            $paymentPayload["hash"] = $gateway->signature()->calculateWidgetSignature($paymentPayload);
            $ch = curl_init($gateway->getConfig()->getApiBaseUrl() . '/api/v1/deliveries');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($paymentPayload));
            $response = json_decode(curl_exec($ch), true);
            print_r($response);
            exit;
        }
        
        exit("OK");
    }
}