<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\PaymentModule;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;

class Magento_Paystack extends PaymentModule
{
    /**
    * Paystack configuration
    */
    const MAGENTO_PAYSTACK_ENABLED = 'MAGENTO_PAYSTACK_ENABLED';
    const MAGENTO_PAYSTACK_CHECKOUT_URL = 'MAGENTO_PAYSTACK_CHECKOUT_URL';

    public function __construct()
    {
        $this->name = 'magento_paystack';
        $this->icon = 'fas fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Paystack';
        $this->description = 'Magento Paystack allows your customers to checkout Paystack on a different website.';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::MAGENTO_PAYSTACK_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function makeRequest(
        $site_id, 
        $handle, 
        $node_id, 
        $customer_id, 
        $email, 
        $currency, 
        $amount, 
        $redirectUri, 
        $success_url, 
        $decline_url
    ) {
        $app = $this->application;
        
        $checkoutUrl = Configuration::get(self::MAGENTO_PAYSTACK_CHECKOUT_URL);
        $checkoutUrl = rtrim($checkoutUrl,"/");

        $data = array(
            'site_id' => $site_id,
            'node_id' => $node_id,
            'customer_id' => $customer_id,
            'email' => $email,
            'handle' => $handle,
            'payment_method' => $this->name,
            'amount' => $amount,
            'currency' => $currency,
            'cancel_url' => $redirectUri,
            'success_url' => $success_url,
            'decline_url' => $decline_url
        );

        $ch = curl_init();
        curl_setopt($ch,CURLOPT_URL, $checkoutUrl . '/authorize' );
        curl_setopt($ch,CURLOPT_POST, 1);                
        curl_setopt($ch,CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_CONNECTTIMEOUT ,3);
        curl_setopt($ch,CURLOPT_TIMEOUT, 20);
        $response = curl_exec($ch);

        $response = Tools::jsonDecode($response, true);

        if(curl_error($ch) || !ArrayUtils::get($response, 'cart_id')) {
            header('location:' . $decline_url);
            exit;
        }

        curl_close($ch);

        $cart_id = ArrayUtils::get($response, 'cart_id');
        header('location:' . $checkoutUrl . '/' . $cart_id);
        exit();
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $user = $app->container->user;

        $redirectUri = $app->base_uri . '/dashboard/order/' . $order->id;
        $decline_url = $app->base_uri . '/payment/'.$this->name.'/failed?handle=order&redirectUri=' . $redirectUri;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?handle=order&redirectUri=' . $redirectUri;
       
        $this->makeRequest(
            $order->site_id, 
            'order',
            $order->id, 
            $user->id, 
            $user->email, 
            $currency->iso_code, 
            toFixed($order->total), 
            $redirectUri, 
            $success_url, 
            $decline_url
        );
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $user = $app->container->user;

        $redirectUri = $app->base_uri . '/dashboard/paper/' . $paper->id;
        $decline_url = $app->base_uri . '/payment/'.$this->name.'/failed?handle=paper&redirectUri=' . $redirectUri;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?handle=paper&redirectUri=' . $redirectUri;
       
        $this->makeRequest(
            PROX_SITE_ID, 
            'paper',
            $paper->id, 
            $user->id, 
            $user->email, 
            $currency->iso_code, 
            toFixed($paper->price), 
            $redirectUri, 
            $success_url, 
            $decline_url
        );
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $user = $app->container->user;

        $redirectUri = $app->base_uri . '/dashboard/add-payment/' . $addPayment->id;
        $succesRedirectUri = $app->base_uri . '/dashboard/order/' . $addPayment->order_id;
        $decline_url = $app->base_uri . '/payment/'.$this->name.'/failed?handle=add-payment&redirectUri=' . $redirectUri;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?handle=add-payment&redirectUri=' . $succesRedirectUri;
       
        $this->makeRequest(
            PROX_SITE_ID, 
            'add-payment',
            $addPayment->id, 
            $user->id, 
            $user->email, 
            $currency->iso_code, 
            toFixed($addPayment->total), 
            $redirectUri, 
            $success_url, 
            $decline_url
        );
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $user = $app->container->user;

        $redirectUri = $app->base_uri . '/dashboard/wallet';
        $decline_url = $app->base_uri . '/payment/'.$this->name.'/failed?handle=wallet&redirectUri=' . $redirectUri;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?handle=wallet&redirectUri=' . $redirectUri;
       
        $this->makeRequest(
            PROX_SITE_ID, 
            'wallet',
            $user_id, 
            $user->id, 
            $user->email, 
            $currency->iso_code, 
            toFixed($amount), 
            $redirectUri, 
            $success_url, 
            $decline_url
        );
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $user = $app->container->user;

        $redirectUri = $app->base_uri . '/dashboard/order/' . $order_id;
        $decline_url = $app->base_uri . '/payment/'.$this->name.'/failed?handle=tip&redirectUri=' . $redirectUri;
        $success_url = $app->base_uri . '/payment/'.$this->name.'/success?handle=tip&redirectUri=' . $redirectUri;
       
        $this->makeRequest(
            PROX_SITE_ID, 
            'tip',
            $order_id, 
            $user->id, 
            $user->email, 
            'USD', 
            toFixed($amount), 
            $redirectUri, 
            $success_url, 
            $decline_url
        );
    } 

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;

        $paymentId = ArrayUtils::get($params, 'paymentId');

        $checkoutUrl = Configuration::get(self::MAGENTO_PAYSTACK_CHECKOUT_URL);
        $checkoutUrl = rtrim($checkoutUrl,"/");

        $ch = curl_init();
        curl_setopt($ch,CURLOPT_URL, $checkoutUrl . '/payment/' . $paymentId );
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);

        if(curl_errno($ch)) {
            $smarty->assign(array(
                'status' => 'error'
            ));
        } 

        curl_close($ch);

        $payment = Tools::jsonDecode($response, true);

        if(ArrayUtils::get($payment, 'node_id')) {
            $handle = ArrayUtils::get($payment, 'handle');
            $nodeId = (int) ArrayUtils::get($payment, 'node_id');
            $params['paymentId'] = ArrayUtils::get($payment, 'payment_code');
            $params['payment_method'] = ArrayUtils::get($payment, 'payment_method');
            $params['customer_id'] = ArrayUtils::get($payment, 'customer_id');
            $params['currency'] = ArrayUtils::get($payment, 'currency');
            $params['amount'] = (float) ArrayUtils::get($payment, 'amount');

            PaymentModule::processPayment( $nodeId, $handle, $params );

            $smarty->assign(array(
                'status' => 'success'
            ));
        } else {
            $smarty->assign(array(
                'status' => 'error'
            ));
        }
    }
}