<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class Paystack extends PaymentModule
{
    /**
    * Paystack configuration
    */
    const PAYSTACK_ENABLED = 'PAYSTACK_ENABLED';
    const PAYSTACK_MODE = 'PAYSTACK_MODE';
    const PAYSTACK_PUBLIC_KEY = 'PAYSTACK_PUBLIC_KEY';
    const PAYSTACK_SECRET_KEY = 'PAYSTACK_SECRET_KEY';

    public function __construct()
    {
        $this->name = 'paystack';
        $this->icon = 'fab fa-paystack';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Paystack';
        $this->description = 'Set up Paystack as a payment method and enable users to pay without leaving your website';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::PAYSTACK_ENABLED);
    }

    public function install()
    { 
        return parent::install() && 
        $this->registerHook('paymentOptions') &&
        $this->registerHook('verifyPayment') &&
        $this->registerHook('displayClientHeader') &&
        $this->registerHook('displayClientFooter');
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency, $checkoutOptions = array()) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'amount_100' => toFixed($order->total*100),
            'paystack_mode' => Configuration::get( self::PAYSTACK_MODE ),
            'paystack_public_key' => Configuration::get( self::PAYSTACK_PUBLIC_KEY ),
            'paystack_secret' => Configuration::get( self::PAYSTACK_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paystack.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'amount_100' => toFixed($paper->price*100),
            'paystack_mode' => Configuration::get( self::PAYSTACK_MODE ),
            'paystack_public_key' => Configuration::get( self::PAYSTACK_PUBLIC_KEY ),
            'paystack_secret' => Configuration::get( self::PAYSTACK_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paystack.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'amount_100' => toFixed($addPayment->total*100),
            'paystack_mode' => Configuration::get( self::PAYSTACK_MODE ),
            'paystack_public_key' => Configuration::get( self::PAYSTACK_PUBLIC_KEY ),
            'paystack_secret' => Configuration::get( self::PAYSTACK_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paystack.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'amount_100' => toFixed($amount*100),
            'paystack_mode' => Configuration::get( self::PAYSTACK_MODE ),
            'paystack_public_key' => Configuration::get( self::PAYSTACK_PUBLIC_KEY ),
            'paystack_secret' => Configuration::get( self::PAYSTACK_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paystack.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'amount_100' => toFixed($amount*100),
            'paystack_mode' => Configuration::get( self::PAYSTACK_MODE ),
            'paystack_public_key' => Configuration::get( self::PAYSTACK_PUBLIC_KEY ),
            'paystack_secret' => Configuration::get( self::PAYSTACK_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paystack.tip');
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */ 
    public function verifyPayment($node_id, $handle, $payload) 
    { 
        $app = $this->application;
        
        $payment_method = ArrayUtils::get($payload, 'payment_method');
        $reference = ArrayUtils::get($payload, 'reference');

        $secretKey = Configuration::get( self::PAYSTACK_SECRET_KEY );

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.paystack.co/transaction/verify/" . $reference,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
                "Authorization: Bearer " . $secretKey,
                "Cache-Control: no-cache",
            ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);
        
        if ($err) {
            $app->response([
                'error' => true,
            ]);
        }
        
        $response = Tools::jsonDecode($response, true);

        if(!ArrayUtils::has($response, 'data')) {
            $app->response([
                'error' => true,
            ]);
        }

        $data = ArrayUtils::get($response, 'data');
        $status = ArrayUtils::get($data, 'status');
        $amount = (float) ArrayUtils::get($data, 'amount');

        if($status == 'success') {
            $payload['paymentId'] = $reference;
            $payload['nodeId'] = $node_id;
            $payload['handle'] = $handle;
            $payload['amount'] = $amount/100;

            PaymentModule::processPayment( $node_id, $handle, $payload );

            $app->response([
                'success' => true,
            ]);
        } else {
            $app->response([
                'error' => true,
            ]);
        }
    }
}