<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class PeachPayments extends PaymentModule
{
    /**
    * PeachPayments configuration
    */
    const PEACHPAYMENTS_ENABLED = 'PEACHPAYMENTS_ENABLED';
    const PEACHPAYMENTS_MODE = 'PEACHPAYMENTS_MODE';
    const PEACHPAYMENTS_ENTITY_ID = 'PEACHPAYMENTS_ENTITY_ID';
    const PEACHPAYMENTS_ACCESS_TOKEN = 'PEACHPAYMENTS_ACCESS_TOKEN';

    const PEACHPAYMENT_REQUEST_SUCCESSFULLY_PROCESSED = "/^(000\.400\.0[^3]|000\.400\.100)/";			
    const PEACHPAYMENT_TRANSACTION_SUCCEEDED = "/^(000\.000\.|000\.100\.1|000\.[36])/";

    public $checkout_url;

    public function __construct()
    {
        $this->name = 'peachpayments';
        $this->icon = 'fab fa-peachpayments';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'Peach Payments';
        $this->description = 'PeachPayment Payment Gateway allows you to accept payment on your site via credit card';

        if(Configuration::get( self::PEACHPAYMENTS_MODE ) == 'live') {
            $this->checkout_url = 'https://oppwa.com/v1/checkouts';
        } else {
            $this->checkout_url = 'https://test.oppwa.com/v1/checkouts';
        }
    }

    public function isActive() {
        return Configuration::get(self::PEACHPAYMENTS_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function setVars() {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'peachpayments_mode' => Configuration::get( self::PEACHPAYMENTS_MODE ),
            'peachpayments_entity_id' => Configuration::get( self::PEACHPAYMENTS_ENTITY_ID ),
            'peachpayments_access_token' => Configuration::get( self::PEACHPAYMENTS_ACCESS_TOKEN ),
            'payee_email' => $user->email
        ]);
    }

    public function createCheckoutId( $invoice_id, $total, $iso_code ) {
        $entityId = Configuration::get( self::PEACHPAYMENTS_ENTITY_ID );
        $accessToken = Configuration::get( self::PEACHPAYMENTS_ACCESS_TOKEN );

        $data = "entityId=" . $entityId . "&merchantInvoiceId=" . $invoice_id . "&amount=" . $total . "&currency=" . $iso_code . "&paymentType=DB";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->checkout_url );
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization:Bearer " . $accessToken ));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $checkoutResult = curl_exec($ch);

        if(curl_errno($ch)) {
            throw new Exception("Something went wrong");
        }
        curl_close($ch);

        $checkoutData = Tools::jsonDecode($checkoutResult, true);
        return ArrayUtils::get($checkoutData, 'id');
    }

    public function checkStatus( $paymentId ) {
        $entityId = Configuration::get( self::PEACHPAYMENTS_ENTITY_ID );
        $accessToken = Configuration::get( self::PEACHPAYMENTS_ACCESS_TOKEN );

        $url = $this->checkout_url . '/' . $paymentId . '/payment?entityId=' . $entityId;
    
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization:Bearer ' . $accessToken));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);// this should be set to true in production
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $responseData = curl_exec($ch);
        if(curl_errno($ch)) {
            return curl_error($ch);
        }
        curl_close($ch);

        $responseData = Tools::jsonDecode($responseData);

        return $responseData ;
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $this->setVars();

        $checkout_id = $this->createCheckoutId($order->id, $order->total, $currency->iso_code);
        $smarty->assign([
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id,
            'callback_url' => $app->base_uri . '/payment/'.$this->name.'/callback?handle=order',
            'checkout_id' => $checkout_id
        ]);
        
        return $this->getTemplateContent('peachpayments.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $this->setVars();

        $checkout_id = $this->createCheckoutId($paper->id, $paper->total, $currency->iso_code);
        $smarty->assign([
            'redirectUri' => $app->base_uri . '/dashboard/paper/' . $paper->id,
            'callback_url' => $app->base_uri . '/payment/' . $this->name.'/callback?handle=paper',
            'checkout_id' => $checkout_id
        ]);
        return $this->getTemplateContent('peachpayments.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $this->setVars();

        $checkout_id = $this->createCheckoutId($addPayment->id, $addPayment->total, $currency->iso_code);
        $smarty->assign([
            'redirectUri' => $app->base_uri . '/dashboard/add-payment/' . $addPayment->id,
            'callback_url' => $app->base_uri . '/payment/'.$this->name.'/callback?handle=add-payment',
            'checkout_id' => $checkout_id
        ]);
        return $this->getTemplateContent('peachpayments.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $this->setVars();

        $checkout_id = $this->createCheckoutId($user_id, $amount, $currency->iso_code);
        $smarty->assign([
            'redirectUri' => $app->base_uri . '/dashboard/wallet',
            'callback_url' => $app->base_uri . '/payment/'.$this->name.'/callback?handle=wallet',
            'checkout_id' => $checkout_id
        ]);
        return $this->getTemplateContent('peachpayments.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $this->setVars();

        $checkout_id = $this->createCheckoutId($order_id, $amount, $currency->iso_code);
        $smarty->assign([
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order_id,
            'callback_url' => $app->base_uri . '/payment/'.$this->name.'/callback?handle=tip',
            'checkout_id' => $checkout_id
        ]);

        return $this->getTemplateContent('peachpayments.tip');
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $handle = ArrayUtils::get($params, 'handle');
        $node_id = (int) ArrayUtils::get($params, 'nodeId');
        $id = ArrayUtils::get($params, 'id');
        $resourcePath = ArrayUtils::get($params, 'resourcePath');
    
        $response = $this->checkStatus($id);

        $params['customer_id'] = $user->id;
        $params['payment_method'] = $this->name;
        $params['amount'] = $response->amount;
        $params['paymentId'] = $id;

        if ( preg_match(self::PEACHPAYMENT_REQUEST_SUCCESSFULLY_PROCESSED, $response->result->code) || preg_match(self::PEACHPAYMENT_TRANSACTION_SUCCEEDED, $response->result->code)) {
            $smarty->assign(array(
                'status' => 'success'
            ));
            PaymentModule::processPayment( $node_id, $handle, $params );
        } else {
            $smarty->assign(array(
                'status' => 'error'
            ));
        }
    }
}