<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\User\Customer;
use Proxim\User\Employee;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;
use Twilio\Rest\Client;

require_once dirname(__FILE__) . '/vendor/autoload.php';

class SMSNotifications extends Module
{
    const SMS_NOTIFICATIONS_SMS_PROVIDER = 'SMS_NOTIFICATIONS_SMS_PROVIDER';
    const SMS_NOTIFICATIONS_TWILIO_SID = 'SMS_NOTIFICATIONS_TWILIO_SID';
    const SMS_NOTIFICATIONS_TWILIO_TOKEN = 'SMS_NOTIFICATIONS_TWILIO_TOKEN';
    const SMS_NOTIFICATIONS_TWILIO_PHONE = 'SMS_NOTIFICATIONS_TWILIO_PHONE';
    const SMS_NOTIFICATIONS_BULKSMS_USERNAME = 'SMS_NOTIFICATIONS_BULKSMS_USERNAME';
    const SMS_NOTIFICATIONS_BULKSMS_PASSWORD = 'SMS_NOTIFICATIONS_BULKSMS_PASSWORD';
    const SMS_NOTIFICATIONS_INFOBIP_USERNAME = 'SMS_NOTIFICATIONS_INFOBIP_USERNAME';
    const SMS_NOTIFICATIONS_INFOBIP_PASSWORD = 'SMS_NOTIFICATIONS_INFOBIP_PASSWORD';

    const SMS_NOTIFICATIONS_ORDER_PAYMENT = 'SMS_NOTIFICATIONS_ORDER_PAYMENT';
    const SMS_NOTIFICATIONS_PASSWORD_CHANGE = 'SMS_NOTIFICATIONS_PASSWORD_CHANGE';
    const SMS_NOTIFICATIONS_WRITER_NEW_ORDER = 'SMS_NOTIFICATIONS_WRITER_NEW_ORDER';
    const SMS_NOTIFICATIONS_ORDER_ASSIGN = 'SMS_NOTIFICATIONS_ORDER_ASSIGN';
    const SMS_NOTIFICATIONS_ORDER_MESSAGE = 'SMS_NOTIFICATIONS_ORDER_MESSAGE';
    const SMS_NOTIFICATIONS_FILE_DELETE_REQUEST = 'SMS_NOTIFICATIONS_FILE_DELETE_REQUEST';
    const SMS_NOTIFICATIONS_ORDER_REVISION = 'SMS_NOTIFICATIONS_ORDER_REVISION';
    const SMS_NOTIFICATIONS_ORDER_REFUND = 'SMS_NOTIFICATIONS_ORDER_REFUND';

    public function __construct()
    {
        $this->name = 'smsnotifications';
        $this->icon = 'fas fa-comment-alt';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'SMS Notifications';
        $this->description = 'Send SMS Messages, Password Resets, Alerts, Order Notifications With Confidence';
    }

    public function checkAccess() {
        $user = $this->application->container->user;
        return $user->is_admin || $user->is_sub_admin ? true : false;
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        $this->registerHook([
            'actionEmailSendBefore',
            'actionMailAlterMessageBeforeSend'
        ]);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    /**
     * sendSMS
     *
     * @param string $phone
     * @param string $message
     * @return boolean
     */
    public function sendSMS($phone, $message, $siteId) {
        $smsProvider = Configuration::get(self::SMS_NOTIFICATIONS_SMS_PROVIDER, PROX_ADMIN_SITE_ID);

        switch($smsProvider) {
            case 'twilio':
                try {
                    $client = new Client(
                        Configuration::get(self::SMS_NOTIFICATIONS_TWILIO_SID, PROX_ADMIN_SITE_ID),
                        Configuration::get(self::SMS_NOTIFICATIONS_TWILIO_TOKEN, PROX_ADMIN_SITE_ID)
                    );

                    $message = $client->account->messages->create(
                        $phone,
                        array(
                            'from' => Configuration::get(self::SMS_NOTIFICATIONS_TWILIO_PHONE, PROX_ADMIN_SITE_ID),
                            'body' => $message
                        )
                    );

                    if(!$message->sid) {
                        return false;
                    }
                } catch(Exception $e) {
                    return false;
                }

                break;
            
            case 'bulksms':
                try {
                    $username = Configuration::get('SMS_NOTIFICATIONS_BULKSMS_USERNAME');
                    $password = Configuration::get('SMS_NOTIFICATIONS_BULKSMS_PASSWORD');
                    $messages = array(
                        array(
                            'to' => $phone,
                            'body' => $message
                        )
                    );
                    $ch = curl_init();
                    $headers = array(
                        'Content-Type:application/json',
                        'Authorization:Basic '. base64_encode("$username:$password")
                    );
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                    curl_setopt ( $ch, CURLOPT_URL, "https://api.bulksms.com/v1/messages?auto-unicode=true&longMessageMaxParts=30" );
                    curl_setopt ( $ch, CURLOPT_POST, 1 );
                    curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, 1 );
                    curl_setopt ( $ch, CURLOPT_POSTFIELDS, json_encode($messages) );
                    /* allow cUrl functions 20 seconds to execute */
                    curl_setopt ( $ch, CURLOPT_TIMEOUT, 20 );
                    /* wait 10 seconds while trying to connect */
                    curl_setopt ( $ch, CURLOPT_CONNECTTIMEOUT, 10 );
                    $output = array();
                    $output['server_response'] = curl_exec( $ch );
                    $curl_info = curl_getinfo( $ch );
                    $output['http_status'] = $curl_info[ 'http_code' ];
                    $output['error'] = curl_error($ch);
                    curl_close( $ch );
                    if($output['http_status'] != 201) {
                        return false;
                    }
                } catch(Exception $e) {
                    return false;
                }
                break;
    
            case 'infobip':
                try {
                    $infobip_username = Configuration::get('SMS_NOTIFICATIONS_INFOBIP_USERNAME');
                    $infobip_password = Configuration::get('SMS_NOTIFICATIONS_INFOBIP_PASSWORD');

                    $sms = array(
                        "from" => Configuration::get('SITE_NAME', $siteId),
                        "to" => $phone,
                        "text" => $message
                    );
                    $ch = curl_init();
                    $header = array("Content-Type:application/json", "Accept:application/json");
                    curl_setopt($ch, CURLOPT_URL, "https://api.infobip.com/sms/1/text/single");
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
                    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
                    curl_setopt($ch, CURLOPT_USERPWD, $infobip_username . ":" . $infobip_password );
                    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
                    curl_setopt($ch, CURLOPT_MAXREDIRS, 2);
                    curl_setopt($ch, CURLOPT_POST, 1);
                    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($sms));
                    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                    $response = curl_exec($ch);
                    $http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    curl_close($ch);
                    if(!($http_status >= 200 && $http_status < 300)) {
                        return false;
                    }
                } catch(Exception $e) {
                    return false;
                }

                break;
        }

        return true;
    }

    public function hookActionEmailSendBefore($payload) {
        $to = ArrayUtils::get($payload, 'to');
        $templateVars = ArrayUtils::get($payload, 'templateVars');
        $template = ArrayUtils::get($payload, 'template');
        $siteId = ArrayUtils::get($payload, 'siteId');

        switch($template) {
            case 'payment':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'order_url');
                $payment_method = ArrayUtils::get($templateVars, 'payment_method');
                $paid_at = ArrayUtils::get($templateVars, 'paid_at');
                $paid_amount = ArrayUtils::get($templateVars, 'paid_amount');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_ORDER_PAYMENT, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $customer = new Customer();
                $customer->getByEmail($to, null, $siteId);
                if(Validate::isLoadedObject($customer) && Validate::isPhoneNumber($customer->phone)) {
                    $smsMessage = str_replace(
                        array('{order_id}', '{order_url}', '{payment_method}', '{paid_at}', '{paid_amount}', '{username}'), 
                        array($order_id, $order_url, $payment_method, $paid_at, $paid_amount, $customer->name), 
                        $smsMessage
                    );
                    $this->sendSMS($customer->phone, $smsMessage, $siteId);
                }

                break;

            case 'order_assign':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'order_url');
                $username = ArrayUtils::get($templateVars, 'username');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_ORDER_ASSIGN, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $smsMessage = str_replace(array('{order_id}', '{username}', '{order_url}'), array($order_id, $username, $order_url), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }

                break;

            case 'new_order':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'admin_order_url');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_WRITER_NEW_ORDER, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $smsMessage = str_replace(array('{order_id}', '{order_url}'), array($order_id, $order_url), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }
                break;

            case 'order_revision':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'order_url');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_ORDER_REVISION, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $username = $employee->first_name . ' ' . $employee->last_name;
                    $smsMessage = str_replace(array('{order_id}', '{order_url}, {username}'), array($order_id, $order_url, $username), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }
                break;

            case 'order_message':
                $order_id = ArrayUtils::get($templateVars, 'orderId');
                $subject = ArrayUtils::get($templateVars, 'subject');
                $text_message = ArrayUtils::get($templateVars, 'text_message');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_ORDER_MESSAGE, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $username = $employee->first_name . ' ' . $employee->last_name;
                    $smsMessage = str_replace(array('{order_id}', '{subject}', '{text_message}', '{username}'), array($order_id, $subject, $text_message, $username), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }
                break;

            case 'order_refund':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'order_url');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_ORDER_REFUND, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $smsMessage = str_replace(array('{order_id}', '{order_url}'), array($order_id, $order_url), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }
                break;

            case 'file_delete':
                $order_id = ArrayUtils::get($templateVars, 'order_id');
                $order_url = ArrayUtils::get($templateVars, 'order_url');
                $file_name = ArrayUtils::get($templateVars, 'file_name');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_FILE_DELETE_REQUEST, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $employee = new Employee();
                $employee->getByEmail($to);
                if(Validate::isLoadedObject($employee) && Validate::isPhoneNumber($employee->phone)) {
                    $smsMessage = str_replace(array('{order_id}', '{file_name}', '{order_url}'), array($order_id, $file_name, $order_id), $smsMessage);
                    $this->sendSMS($employee->phone, $smsMessage, $siteId);
                }
                break;

            case 'password_change':
                $username = ArrayUtils::get($templateVars, 'username');
                $confirm_url = ArrayUtils::get($templateVars, 'url');
                $smsMessage = Configuration::get(self::SMS_NOTIFICATIONS_PASSWORD_CHANGE, PROX_ADMIN_SITE_ID);
                if(!$smsMessage) return;

                $customer = new Customer();
                $customer->getByEmail($to, null, $siteId);
                if(Validate::isLoadedObject($customer) && Validate::isPhoneNumber($customer->phone)) {
                    $smsMessage = str_replace(array('{username}', '{confirm_url}'), array($username, $confirm_url), $smsMessage);
                    $this->sendSMS($customer->phone, $smsMessage, $siteId);
                }

                break;
        }
    } 
}
