<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Order\AdditionalService;
use Proxim\Order\Order;
use Proxim\Site\Site;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class Stripe extends PaymentModule
{
    /**
    * Stripe configuration
    */
    const STRIPE_ENABLED = 'STRIPE_ENABLED';
    const STRIPE_JS_CHECKOUT = 'STRIPE_JS_CHECKOUT';
    const STRIPE_MODE = 'STRIPE_MODE';
    const STRIPE_LIVE_KEY = 'STRIPE_LIVE_KEY';
    const STRIPE_LIVE_SECRET = 'STRIPE_LIVE_SECRET';
    const STRIPE_TEST_KEY = 'STRIPE_TEST_KEY';
    const STRIPE_TEST_SECRET = 'STRIPE_TEST_SECRET';

    public function __construct()
    {
        $this->name = 'stripe';
        $this->icon = 'fab fa-stripe';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Credit Card';
        $this->description = 'Set up Stripe as a payment method and enable users to pay without leaving your website';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::STRIPE_ENABLED);
    }

    public function isEUCountry($countryCode) {
        // List of EU country codes
        $euCountries = ['AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'ES', 'FI', 'FR', 'GR', 'HR', 'HU', 'IE', 'IT', 'LT', 'LU', 'LV', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK'];
    
        // Check if the provided country code is in the EU
        return in_array($countryCode, $euCountries);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'stripe_js' => (int) Configuration::get(self::STRIPE_JS_CHECKOUT),
            'stripe_key' => Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_KEY) : Configuration::get(self::STRIPE_TEST_KEY),
        ]);
        
        return $this->getTemplateContent('stripe.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'stripe_js' => (int) Configuration::get(self::STRIPE_JS_CHECKOUT),
            'stripe_key' => Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_KEY) : Configuration::get(self::STRIPE_TEST_KEY),
        ]);
        
        return $this->getTemplateContent('stripe.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'stripe_js' => (int) Configuration::get(self::STRIPE_JS_CHECKOUT),
            'stripe_key' => Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_KEY) : Configuration::get(self::STRIPE_TEST_KEY),
        ]);
        
        return $this->getTemplateContent('stripe.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'stripe_js' => (int) Configuration::get(self::STRIPE_JS_CHECKOUT),
            'stripe_key' => Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_KEY) : Configuration::get(self::STRIPE_TEST_KEY),
        ]);
        
        return $this->getTemplateContent('stripe.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'stripe_js' => (int) Configuration::get(self::STRIPE_JS_CHECKOUT),
            'stripe_key' => Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_KEY) : Configuration::get(self::STRIPE_TEST_KEY),
        ]);
        
        return $this->getTemplateContent('stripe.tip');
    } 

    /**
     * return_json
     * 
     * @param array $response
     * @return json
     */
    public function returnJson($response = array()) {
        header('Content-Type: application/json');
        exit(json_encode($response));
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPayment($node_id, $handle, $payload)
    {
        $app = $this->application;
        $cookie = $app->container->cookie;
        $payload['nodeId'] = $node_id;
        $payload['handle'] = $handle;

        $stripe_token = ArrayUtils::get($payload, 'stripeToken');
        $idempotency_key = ArrayUtils::get($payload, 'idempotency_key');
        $stripe_key = Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_SECRET) : Configuration::get(self::STRIPE_TEST_SECRET);

        switch ($handle) {
            case 'order':
                $redirectUri = $app->base_uri . '/dashboard/orders';

                $order = new Order($node_id);
                if (!Validate::isLoadedObject($order)) {
                    header('Location: ' . $app->base_uri . '/payment/stripe/failed?redirectUri=' . urlencode($redirectUri));
                    exit();
                }
                
                $currency = $order->currency;
                $amount = ($order->total * $currency->conversion_rate) * 100;
                $redirectUri = $app->base_uri . '/dashboard/order/' . $order->id;
                
                $isEuCountry = false;
                if ($cookie->country_code) {
                    $isEuCountry = (bool)$this->isEUCountry($cookie->country_code);
                }
                
                try {
                    \Stripe\Stripe::setApiKey($stripe_key);
                
                    $options['currency'] = [
                        'symbol' => $currency->symbol,
                        'rate' => $currency->conversion_rate
                    ];
                
                    if (Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        $session = \Stripe\Checkout\Session::create([
                            'payment_method_types' => ['card'],
                            'line_items' => [[
                                'price_data' => [
                                    'currency' => $currency->iso_code,
                                    'unit_amount' => floor($amount),
                                    'product_data' => [
                                        'name' => "Payment of " . formatPrice($order->total, $options)
                                    ]
                                ],
                                'quantity' => 1
                            ]],
                            'metadata' => [
                                'handle' => 'order',
                                'customer_id' => (int)$order->customer_id,
                                'nodeId' => $order->id
                            ],
                            'automatic_tax' => [
                                'enabled' => true,
                            ],
                            'mode' => 'payment',
                            'success_url' => sprintf('%s/payment/stripe/success?redirectUri=%s&sessionId={CHECKOUT_SESSION_ID}', $app->base_uri, urlencode($redirectUri)),
                            'cancel_url' => sprintf('%s/payment/stripe/failed?redirectUri=%s', $app->base_uri, urlencode($redirectUri))
                        ]);
                
                        return $this->returnJson([
                            'success' => true,
                            'id' => $session->id
                        ]);
                    } else {
                        $intent = \Stripe\PaymentIntent::create([
                            'payment_method_data' => [
                                'type' => 'card',
                                'card' => [
                                    'token' => $stripe_token
                                ],
                            ],
                            'amount' => floor($amount),
                            'description' => 'O-' . $order->id . ' : ' . base64_encode(Configuration::get('SITE_NAME')),
                            'currency' => $currency->iso_code,
                            'confirmation_method' => 'manual',
                            'confirm' => true,
                        ], [
                            'idempotency_key' => $idempotency_key
                        ]);
                
                        if ($intent->status == 'succeeded') {
                            $payload['paymentId'] = $intent->id;
                            PaymentModule::processPayment($node_id, $handle, $payload);
                            header('Location: ' . $app->base_uri . '/payment/stripe/success?handle=order&nodeId=' . $order->id . '&redirectUri=' . urlencode($redirectUri));
                            exit();
                        } else {
                            header('Location: ' . $app->base_uri . '/payment/stripe/failed?redirectUri=' . urlencode($redirectUri));
                            exit();
                        }
                    }
                } catch (\Exception $e) {
                    $callbackUrl = $app->base_uri . '/payment/stripe/failed?redirectUri=' . urlencode($redirectUri);
                    error_log('Stripe error: ' . $e->getMessage());
                
                    if (Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        return $this->returnJson([
                            'callback' => sprintf('window.location.href="%s";', $callbackUrl)
                        ]);
                    } else {
                        header('Location: ' . $callbackUrl);
                        exit();
                    }
                }


                break;

            case 'paper':
                if(!Module::getInstanceByName('prepapers')) {
                    $app->setTemplate('404');
                    $app->display();
                }

                $customer_id = ArrayUtils::get($payload, 'customer_id');
                $redirectUri = $app->base_uri . '/papers';

                $prepaper = new PrePaper( $node_id );
                if (!Validate::isLoadedObject($prepaper)) {
                    return $app->redirect( $app->base_uri . '/payment/stripe/failed?redirectUri=' . $redirectUri );
                }

                $redirectUri = $app->base_uri.'/dashboard/paper/' . $prepaper->id;

                $amount = $prepaper->price*100;

                try {
                    \Stripe\Stripe::setApiKey($stripe_key);

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        $session = \Stripe\Checkout\Session::create([
                            'payment_method_types' => ['card'],
                            'line_items' => [[
                                'price_data' => [
                                    'currency' => 'USD',
                                    'unit_amount' => floor($amount),
                                    'product_data' => [
                                        'name' => "Payment of " . formatPrice($prepaper->price)
                                    ]
                                ],
                                'quantity' => 1
                            ]],
                            'metadata' => [
                                'handle' => 'paper',
                                'customer_id' => (int) $customer_id,
                                'nodeId' => $prepaper->id
                            ],
                            'mode' => 'payment',
                            'success_url' => sprintf('%s/payment/stripe/success?redirectUri=%s&sessionId={CHECKOUT_SESSION_ID}', $app->base_uri, $redirectUri),
                            'cancel_url' => sprintf('%s/payment/stripe/failed?redirectUri=%s', $app->base_uri, $redirectUri)
                        ]);

                        return $this->returnJson([
                            'success' => true,
                            'id' => $session->id
                        ]);
                    } else {
                        $intent = \Stripe\PaymentIntent::create([
                            'payment_method_data' => [
                                'type' => 'card',
                                'card' => [
                                    'token' => $stripe_token
                                ],
                            ],
                            'description' => 'P-' . $prepaper->id . ' : ' . base64_encode(Configuration::get('SITE_NAME')), 
                            'amount' => floor($amount),
                            'currency' => "USD",
                            'confirmation_method' => 'manual',
                            'confirm' => true,
                        ],
                        [
                            'idempotency_key' => $idempotency_key
                        ]);

                        if ($intent->status == 'succeeded') {
                            $payload['paymentId'] = $intent->id;
                            $payload['customer_id'] = $customer_id;
                            PaymentModule::processPayment( $node_id, $handle, $payload );
                            return $app->redirect($app->base_uri . '/payment/stripe/success?redirectUri=' . $redirectUri);
                        } else {
                            return $app->redirect($app->base_uri . '/payment/stripe/failed?redirectUri=' . $redirectUri);
                        }
                    }
                } catch(\Exception $e) {
                    $callbackUrl = $app->base_uri . '/payment/stripe/failed?redirectUri=' . $redirectUri;

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        return $this->returnJson([
					        'callback' => sprintf('window.location.href="%s";', $callbackUrl)
                        ]);
                    } else {
                        return $app->redirect($callbackUrl);
                    }
                }
                break;

            case 'add-payment':
                $redirectUri = $app->base_uri . '/dashboard/orders';

                $addPayment = new AdditionalService( $node_id );
                if (!Validate::isLoadedObject($addPayment)) {
                    return $app->redirect($app->base_uri.'/payment/stripe/failed?redirectUri='.$redirectUri);
                }

                $order = new Order( (int) $addPayment->order_id );
                if (!Validate::isLoadedObject($order)) {
                    return $app->redirect($app->base_uri.'/payment/stripe/failed?redirectUri='.$redirectUri);
                }

                $currency = $order->currency;
                $amount = ($addPayment->total*$currency->conversion_rate)*100;
                $redirectUri = $app->base_uri . '/dashboard/order/' . $order->id;

                $options['currency'] = array(
                    'symbol' => $currency->symbol,
                    'rate' => $currency->conversion_rate
                );

                try {
                    \Stripe\Stripe::setApiKey($stripe_key);

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        $session = \Stripe\Checkout\Session::create([
                            'payment_method_types' => ['card'],
                            'line_items' => [[
                                'price_data' => [
                                    'currency' => $currency->iso_code,
                                    'unit_amount' => floor($amount),
                                    'product_data' => [
                                        'name' => "Additional Payment of " . formatPrice($addPayment->total, $options)
                                    ]
                                ],
                                'quantity' => 1
                            ]],
                            'metadata' => [
                                'handle' => 'add-payment',
                                'customer_id' => (int) $order->customer_id,
                                'nodeId' => $addPayment->id
                            ],
                            'mode' => 'payment',
                            'success_url' => sprintf('%s/payment/stripe/success?redirectUri=%s&sessionId={CHECKOUT_SESSION_ID}', $app->base_uri, $redirectUri),
                            'cancel_url' => sprintf('%s/payment/stripe/failed?redirectUri=%s', $app->base_uri, $redirectUri)
                        ]);

                        return $this->returnJson([
                            'success' => true,
                            'id' => $session->id
                        ]);
                    } else {
                        $intent = \Stripe\PaymentIntent::create([
                            'payment_method_data' => [
                                'type' => 'card',
                                'card' => [
                                    'token' => $stripe_token
                                ],
                            ],
                            'amount' => floor($amount),
                            'description' => 'AP-' . $order->id . ' : ' . base64_encode(Configuration::get('SITE_NAME')), 
                            'currency' => $currency->iso_code,
                            'confirmation_method' => 'manual',
                            'confirm' => true
                        ],
                        [
                            'idempotency_key' => $idempotency_key
                        ]);

                        if ($intent->status == 'succeeded') {
                            $payload['paymentId'] = $intent->id;
                            PaymentModule::processPayment( $node_id, $handle, $payload );
                            header('Location: ' . $app->base_uri.'/payment/stripe/success?redirectUri=' . $redirectUri);
                            exit();
                        } else {
                            header('Location: ' . $app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri);
                            exit();
                        }
                    }
                } catch(\Exception $e) {
                    $callbackUrl = $app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri;

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        return $this->returnJson([
					        'callback' => sprintf('window.location.href="%s";', $callbackUrl)
                        ]);
                    } else {
                        header('Location: ' . $callbackUrl);
                        exit();
                    }
                }

                break;

            case 'wallet':
                $redirectUri = $app->base_uri . '/dashboard/wallet';

                $customer = new Customer( $node_id );
                if (!Validate::isLoadedObject($customer)) {
                    return $app->redirect( $app->base_uri . '/payment/stripe/failed?redirectUri=' . $redirectUri );
                }

                $amount = ArrayUtils::get($payload, 'amount');
                $currency_code = ArrayUtils::get($payload, 'currency');

                $currency_id = Currency::getIdByIsoCode( $currency_code );
                $currency = new Currency( $currency_id );

                $amountFormatted = ($amount*$currency->conversion_rate)*100;

                $options['currency'] = array(
                    'symbol' => $currency->symbol,
                    'rate' => $currency->conversion_rate
                );

                try {

                    \Stripe\Stripe::setApiKey($stripe_key);

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        $session = \Stripe\Checkout\Session::create([
                            'payment_method_types' => ['card'],
                            'line_items' => [[
                                'price_data' => [
                                    'currency' => $currency->iso_code,
                                    'unit_amount' => floor($amountFormatted),
                                    'product_data' => [
                                        'name' => formatPrice($amount, $options) . " Wallet Deposit"
                                    ]
                                ],
                                'quantity' => 1
                            ]],
                            'metadata' => [
                                'handle' => 'wallet',
                                'nodeId' => $customer->id,
                                'amount' => $amountFormatted
                            ],
                            'mode' => 'payment',
                            'success_url' => sprintf('%s/payment/stripe/success?redirectUri=%s&sessionId={CHECKOUT_SESSION_ID}', $app->base_uri, $redirectUri),
                            'cancel_url' => sprintf('%s/payment/stripe/failed?redirectUri=%s', $app->base_uri, $redirectUri)
                        ]);

                        return $this->returnJson([
                            'success' => true,
                            'id' => $session->id
                        ]);
                    } else {
                        $intent = \Stripe\PaymentIntent::create([
                            'payment_method_data' => [
                                'type' => 'card',
                                'card' => [
                                    'token' => $stripe_token
                                ],
                            ],
                            'description' => 'W-' . $customer->id . ' : ' . base64_encode(Configuration::get('SITE_NAME')), 
                            'amount' => floor($amount),
                            'currency' => $currency->iso_code,
                            'confirmation_method' => 'manual',
                            'confirm' => true
                        ],
                        [
                            'idempotency_key' => $idempotency_key
                        ]);

                        if ($intent->status == 'succeeded') {
                            $payload['paymentId'] = $intent->id;
                            PaymentModule::processPayment( $customer->id, $handle, $payload );
                            return $app->redirect($app->base_uri.'/payment/stripe/success?redirectUri=' . $redirectUri);
                        } else {
                            return $app->redirect($app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri);
                        }
                    }
                } catch(\Exception $e) {
                    $callbackUrl = $app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri;

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        return $this->returnJson([
					        'callback' => sprintf('window.location.href="%s";', $callbackUrl)
                        ]);
                    } else {
                        return $app->redirect($callbackUrl);
                    }
                }

                break;

            case 'tip':
                $redirectUri = $app->base_uri . '/dashboard';

                $order = new Order( $node_id );
                if (!Validate::isLoadedObject($order)) {
                    return $app->redirect( $app->base_uri . '/payment/stripe/failed?redirectUri=' . $redirectUri );
                }

                $redirectUri = $app->base_uri . '/dashboard/order/' . $order->id;

                $amount = ArrayUtils::get($payload, 'amount');
                $currency_code = ArrayUtils::get($payload, 'currency');

                $currency_id = Currency::getIdByIsoCode( $currency_code );
                $currency = new Currency( $currency_id );

                $options['currency'] = array(
                    'symbol' => $currency->symbol,
                    'rate' => $currency->conversion_rate
                );

                try {

                    \Stripe\Stripe::setApiKey($stripe_key);

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        $session = \Stripe\Checkout\Session::create([
                            'payment_method_types' => ['card'],
                            'line_items' => [[
                                'price_data' => [
                                    'currency' => $currency->iso_code,
                                    'unit_amount' => floor($amount*100),
                                    'product_data' => [
                                        'name' => formatPrice($amount, $options) . " Writer Tip"
                                    ]
                                ],
                                'quantity' => 1
                            ]],
                            'metadata' => [
                                'handle' => 'tip',
                                'nodeId' => $order->id,
                                'amount' => $amount*100
                            ],
                            'mode' => 'payment',
                            'success_url' => sprintf('%s/payment/stripe/success?redirectUri=%s&sessionId={CHECKOUT_SESSION_ID}', $app->base_uri, $redirectUri),
                            'cancel_url' => sprintf('%s/payment/stripe/failed?redirectUri=%s', $app->base_uri, $redirectUri)
                        ]);

                        return $this->returnJson([
                            'success' => true,
                            'id' => $session->id
                        ]);
                    } else {
                        $intent = \Stripe\PaymentIntent::create([
                            'payment_method_data' => [
                                'type' => 'card',
                                'card' => [
                                    'token' => $stripe_token
                                ],
                            ],
                            'description' => 'T-' . $order->id . ' : ' . base64_encode(Configuration::get('SITE_NAME')), 
                            'amount' => floor($amount*100),
                            'currency' => $currency->iso_code,
                            'confirmation_method' => 'manual',
                            'confirm' => true
                        ],
                        [
                            'idempotency_key' => $idempotency_key
                        ]);

                        if ($intent->status == 'succeeded') {
                            $payload['paymentId'] = $intent->id;
                            PaymentModule::processPayment( $node_id, $handle, $payload );
                            return $app->redirect($app->base_uri.'/payment/stripe/success?redirectUri=' . $redirectUri);
                        } else {
                            return $app->redirect($app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri);
                        }
                    }

                } catch(\Exception $e) {
                    $callbackUrl = $app->base_uri.'/payment/stripe/failed?redirectUri=' . $redirectUri;

                    if(Configuration::get(self::STRIPE_JS_CHECKOUT)) {
                        return $this->returnJson([
					        'callback' => sprintf('window.location.href="%s";', $callbackUrl)
                        ]);
                    } else {
                        return $app->redirect($callbackUrl);
                    }
                }
                break;
        }
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    { 
        $app = $this->application;
        $cookie = $app->cookie;
        $smarty = $this->smarty;
        $user = $app->container->user;

        if(ArrayUtils::has($params, 'sessionId')) {
            $sessionId = ArrayUtils::get($params, 'sessionId');
            $stripe_key = Configuration::get(self::STRIPE_MODE) == "live" ? Configuration::get(self::STRIPE_LIVE_SECRET) : Configuration::get(self::STRIPE_TEST_SECRET);

            try { 
                $stripe = new \Stripe\StripeClient($stripe_key); 
                $checkoutSession = $stripe->checkout->sessions->retrieve($sessionId);
            } catch(Exception $e) {  
                $smarty->assign([
                    'status' => 'error'
                ]);
            } 

            try { 
                $paymentIntent = $stripe->paymentIntents->retrieve($checkoutSession->payment_intent); 
            } catch (\Stripe\Exception\ApiErrorException $e) { 
                $smarty->assign([
                    'status' => 'error'
                ]);
            } 

            if( $paymentIntent && $paymentIntent->status == 'succeeded' ) {
                $transactionID = $paymentIntent->id; 
                $paidAmount = $paymentIntent->amount; 
                $paidCurrency = $paymentIntent->currency; 

                $metaData = $checkoutSession->metadata;
                $handle = ArrayUtils::get($metaData, 'handle');
                $nodeId = (int) ArrayUtils::get($metaData, 'nodeId');
                $customerId = (int) ArrayUtils::get($metaData, 'customer_id');

                $params['paymentId'] = $transactionID;
                $params['nodeId'] = $nodeId;
                $params['currency'] = $paidCurrency;
                $params['payment_method'] = $this->name;
                $params['customer_id'] = $customerId;
                $params['handle'] = $handle;
                $params['amount'] = ($paidAmount/100);
                
                PaymentModule::processPayment( $nodeId, $handle, $params );
            }

            $smarty->assign(array(
                'status' => 'success' 
            ));
        } else {
            $status = ArrayUtils::get($params, 'status') == 'success' ? 'success' : 'error';
            
            $smarty->assign(array(
                'status' => $status
            ));
        }
    }
}