<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

require_once dirname(__FILE__) . '/classes/DirectPostApi.php';

class T1Payments extends PaymentModule
{
    /**
    * T1Payments configuration
    */
    const T1PAYMENTS_ENABLED = 'T1PAYMENTS_ENABLED';
    const T1PAYMENTS_TOKENIZATION_KEY = 'T1PAYMENTS_TOKENIZATION_KEY';
    const T1PAYMENTS_SECRET_KEY = 'T1PAYMENTS_SECRET_KEY';

    public function __construct()
    {
        $this->name = 't1payments';
        $this->icon = 'fa fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Credit Card';
        $this->description = 'T1 Payments specializes in payment processing for high-risk industries.';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::T1PAYMENTS_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $smarty->assign([
            'css_path' => $this->local_path . 'views/_css.t1payments.tpl',
            'country_code' => $cookie->country_code ? $cookie->country_code : 'US',
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id,
            'tokenization_key' => Configuration::get( self::T1PAYMENTS_TOKENIZATION_KEY )
        ]);

        return $this->getTemplateContent('t1payments.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $smarty->assign([
            'css_path' => $this->local_path . 'views/_css.t1payments.tpl',
            'country_code' => $cookie->country_code ? $cookie->country_code : 'US',
            'redirectUri' => $app->base_uri . '/dashboard/paper/' . $paper->id,
            'tokenization_key' => Configuration::get( self::T1PAYMENTS_TOKENIZATION_KEY )
        ]);
        
        return $this->getTemplateContent('t1payments.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $smarty->assign([
            'css_path' => $this->local_path . 'views/_css.t1payments.tpl',
            'country_code' => $cookie->country_code ? $cookie->country_code : 'US',
            'redirectUri' => $app->base_uri . '/dashboard/add-payment/' . $addPayment->id,
            'tokenization_key' => Configuration::get( self::T1PAYMENTS_TOKENIZATION_KEY )
        ]);
        
        return $this->getTemplateContent('t1payments.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $smarty->assign([
            'css_path' => $this->local_path . 'views/_css.t1payments.tpl',
            'country_code' => $cookie->country_code ? $cookie->country_code : 'US',
            'redirectUri' => $app->base_uri . '/dashboard/wallet',
            'tokenization_key' => Configuration::get( self::T1PAYMENTS_TOKENIZATION_KEY )
        ]);

        return $this->getTemplateContent('t1payments.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;

        $smarty->assign([
            'css_path' => $this->local_path . 'views/_css.t1payments.tpl',
            'country_code' => $cookie->country_code ? $cookie->country_code : 'US',
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order_id,
            'tokenization_key' => Configuration::get( self::T1PAYMENTS_TOKENIZATION_KEY )
        ]);

        return $this->getTemplateContent('t1payments.tip');
    } 

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPayment($node_id, $handle, $payload) 
    {
        $app = $this->application;

        $paymentId = ArrayUtils::get($payload, 'payment_token');
        $amount = (float) ArrayUtils::get($payload, 'amount');
        $currency = ArrayUtils::get($payload, 'currency');
        $email = ArrayUtils::get($payload, 'email');
        $first_name = ArrayUtils::get($payload, 'firstname');
        $last_name = ArrayUtils::get($payload, 'lastname');
        $address1 = ArrayUtils::get($payload, 'address');
        $city = ArrayUtils::get($payload, 'city');
        $zipcode = ArrayUtils::get($payload, 'zipcode');
        $redirectUri = ArrayUtils::get($payload, 'redirectUri');

        $gw = new DirectPostApi( Configuration::get(self::T1PAYMENTS_SECRET_KEY) );
        $gw->doCapture($paymentId, $email, $first_name, $last_name, $node_id, $amount, $currency, $address1, $city, $zipcode);

        if ($gw->responses['response'] == DirectPostApi::APPROVED) {
            $payload['paymentId'] = $paymentId;
            PaymentModule::processPayment( $node_id, $handle, $payload );
            return $app->redirect($app->base_uri . '/payment/t1payments/success?redirectUri=' . $redirectUri);
        } else {
            return $app->redirect($app->base_uri . '/payment/t1payments/failed?redirectUri=' . $redirectUri);
        }
    }
}